
/*
butterworth low pass filter
2 or 4 poles

up to 5 indepenent values can be filtered, index from 0 to 4 inclusive

init with get_biquad(Fc,num_poles,id);
fc is cutoff frequency, from 0.1 to 1.0 but best results from 0.02 to 0.2
num_poles can be 2 up to 4
2 poles is faster

filter a float value with
biquad_filter(float *val,id);
make 'val' a volatile global
*/
#include <math.h>
#define NF 4

float aa[NF][4],d1[NF][4],d2[NF][4];
float w0[NF][4],w1[NF][4],w2[NF][4];
int bqnp[NF];


void get_biquad(float fc, int np, int idx)
  {   // calculate biquad Butterworth L.P. filter coefficients. np = no. of poles, fc = cut off freq., idx = which filter to calc.
  float s,r;
  int i;  
  float a = tan(M_PI*fc);
  float a2 = a*a;
  bqnp[idx] = np;
  for(i=0; i<np; ++i)
    {
    r =  sin(3.141592926 * (2.0*i+1.0)/(4.0*(float)np));
    s = a2 + 2.0*a*r + 1.0;
    aa[i][idx] = a2/s;
    d1[i][idx] = 2.0*(1-a2)/s;
    d2[i][idx] = -(a2 - 2.0*a*r + 1.0)/s;
    w1[i][idx]=w2[i][idx]=w0[i][idx]=0.0;
    }
  }

void biquad_filter(float *val, int idx)
{ // do LP filter, taking input value pointer val, applying filter index by idx, store filtered result
  float s;
  int i;
  s = *val;
  for(i=0; i<bqnp[idx]; ++i)
    {
    w0[i][idx] = d1[i][idx]*w1[i][idx] + d2[i][idx]*w2[i][idx] + s;
    s = aa[i][idx]*(w0[i][idx] + 2.0*w1[i][idx] + w2[i][idx]);
    w2[i][idx] = w1[i][idx];
    w1[i][idx] = w0[i][idx];
    }
  *val = s;
}
